% Limpiar el espacio de trabajo
clc;
clear;

% Obtencin de las derivadas parciales de las ecuaciones de estados
syms X1 X2 X4 U Br Bp Jr Jp k m l g
A=X2
B=(g*l*m*sin(X1)+Br*X4-2*k*U-Bp*X2)/(m*l^2+Jp)
C=X4
D=(k*U*(2*Jr+m*l^2+Jp)+Jr*Bp*X2-Jr*g*l*m*sin(X1)-Br*X4*(Jr+m*l^2+Jp))/(Jr*(m*l^2+Jp))
jacobian([A, B, C, D], [X1, X2, X4, U])

% Definicin de parmetros
m = 0.13; %kg
m1 = 0.4;  %kg
l = 0.05;   %m
g = 9.81;   %m/s^2
Jp = 3.7957e-05; %Kg*m^2
Jr = 2.22228e-05+4.89043e-06; %Kg*m^2

k = 27.4e-3;  %Nm/A
Cp = 4e-5; %Nm/rpm
Bp = 4e-5; %Nm/rpm
Br = 2e-6; %Nm/rpm

% Condiciones iniciales
%theta0 = 0.00001; %rad
theta0 = deg2rad(10); %rad
% Controlador PID
Kp = 0.3;
Ki = 0.01;
Kd = 0.012;

% Representacin en espacio de estados:
mt = m+m1;
Jl = mt*(l^2)+Jp;

A = [0 1 0 0;... 
   (g*l*mt)/Jl -Bp/Jl 0 Br/Jl;...
   0 0 0 1;...
   -(g*l*mt)/Jl Bp/Jl 0 -(Br*(Jr+Jl))/(Jr*Jl)];
B = [0 -2*k/Jl 0 k*(2*Jr+Jl)/(Jr*Jl)]';
C = [1 0 0 0];
D = 0;

Q = [1 0 0 0;
      0 1 0 0;
      0 0 1 0;
      0 0 0 1];
R = 100;

K = lqr(A,B,Q,R,0)

A1 = [0 1 0;... 
   (g*l*mt)/Jl -Bp/Jl Br/Jl;...
   -(g*l*mt)/Jl Bp/Jl -(Br*(Jr+Jl))/(Jr*Jl)];
B1 = [0 -2*k/Jl k*(2*Jr+Jl)/(Jr*Jl)]';
C1 = [1 0 0];
D1 = 0;

sys = ss(A,B,C,D)

% Control LQR
Q1 = [1 0 0;
      0 1 0;
      0 0 1];
R1 = 100;

K1 = lqr(A1,B1,Q1,R1,0)

sys_cl = ss((A1 - B1*K1),B1,C1,D1);